/*
 * Botendo
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

package de.limited_dev.botendo.extensions.music

import com.kotlindiscord.kord.extensions.commands.Arguments
import com.kotlindiscord.kord.extensions.commands.converters.impl.string
import com.kotlindiscord.kord.extensions.extensions.Extension
import com.kotlindiscord.kord.extensions.extensions.publicSlashCommand
import com.kotlindiscord.kord.extensions.types.respond
import de.limited_dev.botendo.Bot
import de.limited_dev.botendo.extensions.music.components.MusicManager
import de.limited_dev.botendo.util.MessageUtil
import dev.kord.common.Color
import dev.schlaubi.lavakord.audio.Link
import dev.schlaubi.lavakord.kord.getLink


class PlayExtension : Extension() {

    override val name = "play"

    override suspend fun setup() {
        publicSlashCommand(::PlayArgs) {
            name = "play"
            description = "Play music"
            this.action {
                val guildId = this.guild!!.id
                val link = Bot.lava.getLink(guildId)
                val u = this.user
                val vcsUser = u.asMember(guildId).getVoiceStateOrNull()
                if (vcsUser == null) {
                    this.respond {
                        embeds.add(
                            MessageUtil.getEmbed(
                                Color(0xE0311A),
                                "You are not connected to a VC",
                                "Please connect to a VC",
                                u.asUser().username + "#" + u.asUser().discriminator
                            )
                        )
                    }
                    return@action
                }

                val channelId = vcsUser.channelId

                if (link.state != Link.State.CONNECTED) {
                    link.connectAudio(channelId!!.value)
                } else if (link.state == Link.State.CONNECTED && link.lastChannelId != channelId!!.value) {
                    this.respond {
                        embeds.add(
                            MessageUtil.getEmbed(
                                Color(0xE0311A),
                                "You are not in my VC",
                                "We are not in the same VC and therefore, you cannot play any music",
                                u.asUser().username + "#" + u.asUser().discriminator
                            )
                        )
                    }
                    return@action
                }

                val query = arguments.linkquery
                val search = if (query.startsWith("http")) {
                    query
                } else {
                    "ytsearch:$query"
                }
                this.respond {
                    embeds.add(
                        MessageUtil.getEmbed(
                            Color(0xE0A81A),
                            "Searching...",
                            "We are looking for $query",
                            u.asUser().username + "#" + u.asUser().discriminator
                        )
                    )
                }
                MusicManager.addToQueue(this, link, search)
            }
        }
    }

    inner class PlayArgs : Arguments() {
        val linkquery by string {
            name = "linkqery"
            description = "Song link or search query"
        }
    }
}

